package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public class TABLESWITCH extends Select
{
    TABLESWITCH() {
    }
    
    public TABLESWITCH(final int[] match, final InstructionHandle[] targets, final InstructionHandle defaultTarget) {
        super((short)170, match, targets, defaultTarget);
        final short length = (short)(13 + this.getMatchLength() * 4);
        super.setLength(length);
        this.setFixedLength(length);
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitVariableLengthInstruction(this);
        v.visitStackConsumer(this);
        v.visitBranchInstruction(this);
        v.visitSelect(this);
        v.visitTABLESWITCH(this);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        super.dump(out);
        final int matchLength = this.getMatchLength();
        final int low = (matchLength > 0) ? super.getMatch(0) : 0;
        out.writeInt(low);
        final int high = (matchLength > 0) ? super.getMatch(matchLength - 1) : 0;
        out.writeInt(high);
        for (int i = 0; i < matchLength; ++i) {
            out.writeInt(this.setIndices(i, this.getTargetOffset(super.getTarget(i))));
        }
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        super.initFromFile(bytes, wide);
        final int low = bytes.readInt();
        final int high = bytes.readInt();
        final int matchLength = high - low + 1;
        this.setMatchLength(matchLength);
        final short fixedLength = (short)(13 + matchLength * 4);
        this.setFixedLength(fixedLength);
        super.setLength((short)(fixedLength + super.getPadding()));
        super.setMatches(new int[matchLength]);
        super.setIndices(new int[matchLength]);
        super.setTargets(new InstructionHandle[matchLength]);
        for (int i = 0; i < matchLength; ++i) {
            super.setMatch(i, low + i);
            super.setIndices(i, bytes.readInt());
        }
    }
}
